@@ -126,6 +126,28 @@ PyCustomWidgets::PyCustomWidgets(QObject *parent) : QObject(parent),
             if (!library.load())
                 return;
 
+            // If we seem to be running in a venv the set the program name
+            // explicitly so that Python finds the right site-packages.
+            QString venv = QString::fromLocal8Bit(qgetenv("VIRTUAL_ENV"));
+
+            if (!venv.isEmpty())
+            {
+                venv.append(QDir::separator());
+#if defined(Q_OS_WIN)
+                venv.append(QLatin1String("Scripts"));
+#else
+                venv.append(QLatin1String("bin"));
+#endif
+                venv.append(QDir::separator()).append(QLatin1String("python"));
+
+                wchar_t *venv_wc = new wchar_t[venv.length() + 1];
+                venv_wc[venv.toWCharArray(venv_wc)] = L'\0';
+
+                Py_SetProgramName(venv_wc);
+
+                delete venv_wc;
+            }
+
             Py_Initialize();
 
 #ifdef WITH_THREAD
@@ -178,7 +200,6 @@ bool PyCustomWidgets::importPlugins(const QString &dir, const QStringList &plugi
     // Convert the directory to a Python object with native separators.
     QString native_dir = QDir::toNativeSeparators(dir);
 
-#if PY_VERSION_HEX >= 0x03030000
     PyObject *dobj = PyUnicode_FromKindAndData(PyUnicode_2BYTE_KIND, native_dir.constData(), native_dir.length());
 
     if (!dobj)
@@ -186,41 +207,6 @@ bool PyCustomWidgets::importPlugins(const QString &dir, const QStringList &plugi
         PyErr_Print();
         return false;
     }
-#elif PY_VERSION_HEX >= 0x03000000
-#if defined(Py_UNICODE_WIDE)
-    QVector<uint> ucs4 = native_dir.toUcs4();
-
-    PyObject *dobj = PyUnicode_FromUnicode(0, ucs4.size());
-
-    if (!dobj)
-    {
-        PyErr_Print();
-        return false;
-    }
-
-    memcpy(PyUnicode_AS_UNICODE(dobj), ucs4.constData(),
-            ucs4.size() * sizeof (Py_UNICODE));
-#else
-    PyObject *dobj = PyUnicode_FromUnicode(0, native_dir.length());
-
-    if (!dobj)
-    {
-        PyErr_Print();
-        return false;
-    }
-
-    memcpy(PyUnicode_AS_UNICODE(dobj), native_dir.utf16(),
-            native_dir.length() * sizeof (Py_UNICODE));
-#endif
-#else
-    PyObject *dobj = PyString_FromString(native_dir.toLatin1().constData());
-
-    if (!dobj)
-    {
-        PyErr_Print();
-        return false;
-    }
-#endif
 
     // Add the directory to sys.path.
     int rc = PyList_Append(sys_path, dobj);
