@@ -32,7 +32,7 @@
 #include "sipAPIQtCore.h"
 
 
-// The registered Python enum types.
+// The registered user-defined Python enum types.
 QSet<PyObject *> Chimera::_py_enum_types;
 
 // The cache of previously parsed argument type lists.
@@ -156,7 +156,7 @@ Chimera::~Chimera()
 }
 
 
-// Register the type of a Python enum.
+// Register the type of a user-defined Python enum.
 void Chimera::registerPyEnum(PyObject *enum_type)
 {
     Py_INCREF(enum_type);
@@ -233,9 +233,26 @@ const Chimera *Chimera::parse(const QByteArray &name)
 const Chimera *Chimera::parse(const QMetaProperty &mprop)
 {
     Chimera *ct = new Chimera;
-    const char *type_name = mprop.typeName();
+    QByteArray type_name;
 
-    ct->_type = sipFindType(type_name);
+    // The type name of an enum property sometimes includes the scope in the
+    // name and sometimes doesn't (maybe only within the scope itself).
+    // Therefore get the name of the meta-enum.
+    if (mprop.isEnumType())
+    {
+        QMetaEnum menum = mprop.enumerator();
+
+        type_name = QByteArray(menum.name());
+
+        if (menum.scope())
+            type_name.prepend("::").prepend(menum.scope());
+    }
+    else
+    {
+        type_name = QByteArray(mprop.typeName());
+    }
+
+    ct->_type = sipFindType(type_name.constData());
     ct->metatype = mprop.metaType();
     ct->_inexact = true;
     ct->_is_qflags = mprop.isFlagType();
@@ -449,7 +466,11 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
 
         set_qflags(td);
 
-        if (sipTypeIsEnum(td) || _is_qflags)
+        if (sipTypeIsEnum(td))
+        {
+            metatype = QMetaType::fromName(_name);
+        }
+        else if (_is_qflags)
         {
             metatype = QMetaType(QMetaType::Int);
         }
@@ -517,11 +538,10 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
         metatype = QMetaType(QMetaType::VoidStar);
         _name = "void*";
     }
-    else
-    {
-        // Fallback to using a PyQt_PyObject.
+
+    // Fallback to using a PyQt_PyObject.
+    if (metatype.id() == QMetaType::UnknownType)
         metatype = QMetaType::fromType<PyQt_PyObject>();
-    }
 
     // If there is no name so far then use the meta-type name.
     if (_name.isEmpty())
@@ -724,21 +744,25 @@ bool Chimera::parse_cpp_type(const QByteArray &type)
 
     set_qflags(_type);
 
-    if (sipTypeIsClass(_type) && is_ptr)
-    {
-        PyTypeObject *type_obj = sipTypeAsPyTypeObject(_type);
-
-        if (PyType_IsSubtype(type_obj, sipTypeAsPyTypeObject(sipType_QObject)))
-            metatype = QMetaType(QMetaType::QObjectStar);
-    }
-
-    // We don't support pointers to enums.
-    if (sipTypeIsEnum(_type) && is_ptr)
-        _type = 0;
-
-    if (sipTypeIsEnum(_type) || _is_qflags)
+    if (_is_qflags)
         metatype = QMetaType(QMetaType::Int);
 
+    if (is_ptr)
+    {
+        // We don't support pointers to enums.
+        if (sipTypeIsEnum(_type))
+        {
+            _type = 0;
+        }
+        else if (sipTypeIsClass(_type))
+        {
+            PyTypeObject *type_obj = sipTypeAsPyTypeObject(_type);
+
+            if (PyType_IsSubtype(type_obj, sipTypeAsPyTypeObject(sipType_QObject)))
+                metatype = QMetaType(QMetaType::QObjectStar);
+        }
+    }
+
     return true;
 }
 
@@ -773,11 +797,7 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
         break;
 
     case QMetaType::Int:
-        if (isEnum())
-            *reinterpret_cast<int *>(cpp) = sipConvertToEnum(py, _type);
-        else
-            *reinterpret_cast<int *>(cpp) = PyLong_AsLong(py);
-
+        *reinterpret_cast<int *>(cpp) = PyLong_AsLong(py);
         break;
 
     case QMetaType::UInt:
@@ -914,7 +934,11 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
     default:
         if (_type)
         {
-            if (_name.endsWith('*'))
+            if (sipTypeIsEnum(_type))
+            {
+                *reinterpret_cast<int *>(cpp) = sipConvertToEnum(py, _type);
+            }
+            else if (_name.endsWith('*'))
             {
                 // This must be a pointer-type.
 
@@ -1050,16 +1074,7 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
         break;
 
     case QMetaType::Int:
-        if (isEnum())
-        {
-            tmp_storage.tmp_int = sipConvertToEnum(py, _type);
-        }
-        else if (!_inexact)
-        {
-            // It must fit into a C++ int.
-            tmp_storage.tmp_int = PyLong_AsLong(py);
-        }
-        else
+        if (_inexact)
         {
             // Fit it into the smallest C++ type we can.
 
@@ -1108,6 +1123,11 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
                 metatype_used = QMetaType::LongLong;
             }
         }
+        else
+        {
+            // It must fit into a C++ int.
+            tmp_storage.tmp_int = PyLong_AsLong(py);
+        }
 
         break;
 
@@ -1261,7 +1281,11 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
     default:
         if (_type)
         {
-            if (_name.endsWith('*'))
+            if (sipTypeIsEnum(_type))
+            {
+                tmp_storage.tmp_int = sipConvertToEnum(py, _type);
+            }
+            else if (_name.endsWith('*'))
             {
                 ptr_class = sipForceConvertToType(py, _type, 0,
                         SIP_NO_CONVERTORS, 0, &iserr);
@@ -1472,10 +1496,6 @@ PyObject *Chimera::toPyObject(void *cpp) const
         {
             py = sipConvertFromType(cpp, _type, 0);
         }
-        else if (isCppEnum())
-        {
-            py = sipConvertFromEnum(*reinterpret_cast<int *>(cpp), _type);
-        }
         else
         {
             py = PyLong_FromLong(*reinterpret_cast<int *>(cpp));
@@ -1704,10 +1724,10 @@ bool Chimera::to_char32_t(PyObject *py, char32_t &cpp)
 }
 
 
-// Return true if the type is either a C++ or Python enum.
-bool Chimera::isEnum() const
+// Return true if the type is either a C++ or Python enum or flag.
+bool Chimera::isEnumOrFlag() const
 {
-    if (isCppEnum())
+    if (_type && sipTypeIsEnum(_type))
         return true;
 
     if (_is_qflags)
